@@ -7,6 +7,6 @@ static inline struct pthread *__pthread_self()
 
 #define TP_ADJ(p) (p)
 
-#define CANCEL_REG_IP 32
+#define MC_PC gregs[REG_RIP]
 
 #define CANARY canary2
